@@ -4,10 +4,12 @@ from _tkinter import TclError
 from tkinter.ttk import Progressbar
 import ANYstructure_local.optimize as op
 import numpy as np
-import time
+import time, os
 from tkinter import messagebox
 import ANYstructure_local.example_data as test
+from ANYstructure_local.calc_structure import *
 from ANYstructure_local.helper import *
+import ANYstructure_local.example_data as ex
 import copy, pickle
 import ANYstructure_local.calc_structure
 import ANYstructure_local.helper as hlp
@@ -30,9 +32,50 @@ class CreateOptGeoWindow():
             self._load_count = 0
             self._point_dict = test.get_point_dict()
             self._canvas_scale = 20
+
+
             self._line_to_struc = test.get_line_to_struc()
+
             self._opt_frames = {}
             self._active_points = ['point1','point4','point8','point5']
+            self._ML_buckling = dict()  # Buckling machine learning algorithm
+            for name, file_base in zip(['cl SP buc int predictor', 'cl SP buc int scaler',
+                                        'cl SP ult int predictor', 'cl SP ult int scaler',
+                                        'cl SP buc GLGT predictor', 'cl SP buc GLGT scaler',
+                                        'cl SP ult GLGT predictor', 'cl SP ult GLGT scaler',
+                                        'cl UP buc int predictor', 'cl UP buc int scaler',
+                                        'cl UP ult int predictor', 'cl UP ult int scaler',
+                                        'cl UP buc GLGT predictor', 'cl UP buc GLGT scaler',
+                                        'cl UP ult GLGT predictor', 'cl UP ult GLGT scaler',
+                                        'CSR predictor', 'CSR scaler'
+                                        ],
+                                       ["CL_output_cl_buc_predictor_In-plane_support_cl_1_SP",
+                                        "CL_output_cl_buc_scaler_In-plane_support_cl_1_SP",
+                                        "CL_output_cl_ult_predictor_In-plane_support_cl_1_SP",
+                                        "CL_output_cl_ult_scaler_In-plane_support_cl_1_SP",
+                                        "CL_output_cl_buc_predictor_In-plane_support_cl_2,_3_SP",
+                                        "CL_output_cl_buc_scaler_In-plane_support_cl_2,_3_SP",
+                                        "CL_output_cl_ult_predictor_In-plane_support_cl_2,_3_SP",
+                                        "CL_output_cl_ult_scaler_In-plane_support_cl_2,_3_SP",
+                                        "CL_output_cl_buc_predictor_In-plane_support_cl_1_UP",
+                                        "CL_output_cl_buc_scaler_In-plane_support_cl_1_UP",
+                                        "CL_output_cl_ult_predictor_In-plane_support_cl_1_UP",
+                                        "CL_output_cl_ult_scaler_In-plane_support_cl_1_UP",
+                                        "CL_output_cl_buc_predictor_In-plane_support_cl_2,_3_UP",
+                                        "CL_output_cl_buc_scaler_In-plane_support_cl_2,_3_UP",
+                                        "CL_output_cl_ult_predictor_In-plane_support_cl_2,_3_UP",
+                                        "CL_output_cl_ult_scaler_In-plane_support_cl_2,_3_UP",
+                                        "CL_CSR-Tank_req_cl_predictor",
+                                        "CL_CSR-Tank_req_cl_UP_scaler",
+                                        "CL_CSR_plate_cl,_CSR_web_cl,_CSR_web_flange_cl,_CSR_flange_cl_predictor",
+                                        "CL_CSR_plate_cl,_CSR_web_cl,_CSR_web_flange_cl,_CSR_flange_cl_SP_scaler"]):
+                    self._ML_buckling[name] = None
+                    if os.path.isfile(file_base + '.pickle'):
+                        file = open(file_base + '.pickle', 'rb')
+                        from sklearn.neural_network import MLPClassifier
+                        from sklearn.preprocessing import StandardScaler
+                        self._ML_buckling[name] = pickle.load(file)
+                        file.close()
         else:
             self.app = app
             self._load_objects = app._load_dict
@@ -45,6 +88,7 @@ class CreateOptGeoWindow():
             self._opt_frames = {}
             self._active_points = []
             self._root_dir = app._root_dir
+            self._ML_buckling = app._ML_buckling
 
 
         self._opt_structure = {}
@@ -124,6 +168,7 @@ class CreateOptGeoWindow():
         self._new_option_fraction = tk.IntVar()
         self._new_option_panel = tk.IntVar()
 
+
         ent_w = 10
         self._ent_spacing_upper = tk.Entry(self._frame, textvariable=self._new_spacing_upper, width=ent_w)
         self._ent_spacing_lower = tk.Entry(self._frame, textvariable=self._new_spacing_lower, width=ent_w)
@@ -223,6 +268,8 @@ class CreateOptGeoWindow():
                                                                                     y=start_y - 0.6 * dy)
         tk.Label(self._frame, text='Estimated running time for algorithm not calculated.',
                  font='Verdana 9 bold').place(x=start_x, y=start_y + 2.8 * dy)
+        tk.Label(self._frame, text='- Harmonize stiffener spacing for section.', font='Verdana 9 bold')\
+            .place(x=start_x + 5*dx, y=start_y + 2.8 * dy)
         # self._runnig_time_label = tk.Label(self._frame, text='', font='Verdana 9 bold')
         # self._runnig_time_label.place(x=start_x + 2.7 * dx, y=start_y + 2.8 * dy)
         # tk.Label(self._frame, text='seconds ', font='Verdana 9 bold').place(x=start_x + 3.3 * dx, y=start_y + 2.8 * dy)
@@ -249,8 +296,8 @@ class CreateOptGeoWindow():
         self._ent_fl_thk_lower.place(x=start_x + dx * 7, y=start_y + 2 * dy)
 
         # setting default values
-        init_dim = float(50)  # mm
-        init_thk = float(2)  # mm
+        init_dim = float(100)  # mm
+        init_thk = float(5)  # mm
         self._new_delta_spacing.set(init_dim)
         self._new_delta_pl_thk.set(init_thk)
         self._new_delta_web_h.set(init_dim)
@@ -263,7 +310,7 @@ class CreateOptGeoWindow():
         self._new_pl_thk_lower.set(round(10, 5))
         self._new_web_h_upper.set(round(500, 5))
         self._new_web_h_lower.set(round(300, 5))
-        self._new_web_thk_upper.set(round(22, 5))
+        self._new_web_thk_upper.set(round(25, 5))
         self._new_web_thk_lower.set(round(10, 5))
         self._new_fl_w_upper.set(round(250, 5))
         self._new_fl_w_lower.set(round(50, 5))
@@ -311,12 +358,14 @@ class CreateOptGeoWindow():
         # self._new_algorithm_random_trials.trace('w', self.update_running_time)
         # self._new_algorithm.trace('w', self.update_running_time)
 
+
         self.running_time_per_item = 4e-05
         #self._runnig_time_label.config(text=str(self.get_running_time()))
         self._ent_algorithm.place(x=start_x + dx * 10, y=start_y + dy)
         self.algorithm_random_label = tk.Label(self._frame, text='Number of trials')
         tk.Button(self._frame, text='algorith information', command=self.algorithm_info, bg='white') \
             .place(x=start_x + dx * 10, y=start_y + dy * 2)
+
         self.run_button = tk.Button(self._frame, text='RUN OPTIMIZATION!', command=self.run_optimizaion, bg='red',
                                     font='Verdana 10', fg='Yellow')
         self.run_button.place(x=start_x + dx * 10, y=start_y)
@@ -338,6 +387,9 @@ class CreateOptGeoWindow():
         self._new_check_fatigue = tk.BooleanVar()
         self._new_check_slamming = tk.BooleanVar()
         self._new_check_local_buckling = tk.BooleanVar()
+        self._new_harmonize_spacing = tk.BooleanVar()
+        self._new_check_buckling_ml_cl = tk.BooleanVar()
+
         self._new_check_sec_mod.set(True)
         self._new_check_min_pl_thk.set(True)
         self._new_check_shear_area.set(True)
@@ -347,6 +399,10 @@ class CreateOptGeoWindow():
         self._new_check_local_buckling.set(True)
         self._new_option_fraction.set(None)
         self._new_option_panel.set(None)
+        self._new_harmonize_spacing.set(False)
+        self._new_check_buckling_ml_cl.set(False)
+
+        self._new_check_buckling_ml_cl.trace('w', self.update_running_time)
 
         start_y, start_x, dy  = 570, 100, 25
         tk.Label(self._frame,text='Check for minimum section modulus').place(x=start_x+dx*9.7,y=start_y+4*dy)
@@ -356,6 +412,7 @@ class CreateOptGeoWindow():
         tk.Label(self._frame, text='Check for fatigue (RP-C203)').place(x=start_x + dx * 9.7, y=start_y + 8 * dy)
         tk.Label(self._frame, text='Check for bow slamming').place(x=start_x + dx * 9.7, y=start_y + 9 * dy)
         tk.Label(self._frame, text='Check for local stf. buckling').place(x=start_x + dx * 9.7, y=start_y + 10 * dy)
+        tk.Label(self._frame, text='Check for buckling, ML-CL').place(x=start_x + dx * 9.7, y=start_y + 11 * dy)
 
         tk.Label(self._frame, text='Frame (girder data) for weight calculation:', font = 'Verdana 9 bold')\
             .place(x=start_x + dx * 13,
@@ -388,6 +445,11 @@ class CreateOptGeoWindow():
         tk.Checkbutton(self._frame, variable=self._new_check_slamming).place(x=start_x + dx * 12, y=start_y + 9 * dy)
         tk.Checkbutton(self._frame, variable=self._new_check_local_buckling).place(x=start_x + dx * 12,
                                                                                    y=start_y + 10 * dy)
+        tk.Checkbutton(self._frame, variable=self._new_check_buckling_ml_cl).place(x=start_x + dx * 12,
+                                                                                   y=start_y + 11 * dy)
+
+
+        tk.Checkbutton(self._frame, variable=self._new_harmonize_spacing).place(x=start_x + 3.9*dx, y=180)
 
         # Stress scaling
         self._new_fup = tk.DoubleVar()
@@ -426,6 +488,11 @@ class CreateOptGeoWindow():
                                     font='Verdana 10',fg='black')
         self.run_results.place(x=start_x+dx*13, y=start_y - dy * 18)
 
+        self.run_results_prev = tk.Button(self._frame,text='Show previous\n'
+                                                      'results', command=self.show_previous_results, bg='white',
+                                    font='Verdana 10',fg='black')
+        self.run_results_prev.place(x=start_x+dx*15, y=start_y - dy * 20)
+
 
         # ----------------------------------END OF OPTIMIZE SINGLE COPY-----------------------------------------------
         self.progress_count = tk.IntVar()
@@ -508,7 +575,16 @@ class CreateOptGeoWindow():
             self._ent_minstep.place(x=start_x + dx * 15, y=start_y + 0 * dy)
             self._ent_minfunc.place(x=start_x + dx * 15, y=start_y + 1 * dy)
 
-    def run_optimizaion(self, load_pre = False, save_results = True):
+    def show_previous_results(self):
+        # if type(self._geo_results) is not list():
+        #     if self._geo_results is not None:
+        #         return
+        # else:
+        #     if self._geo_results[0] is not None:
+        #         return
+        self.draw_select_canvas(opt_results=self._geo_results)
+
+    def run_optimizaion(self, load_pre = False, save_results = False, harmonize = False):
         '''
         Function when pressing the optimization botton inside this window.
         :return:
@@ -521,7 +597,7 @@ class CreateOptGeoWindow():
         contraints = (self._new_check_sec_mod.get(), self._new_check_min_pl_thk.get(),
                       self._new_check_shear_area.get(), self._new_check_buckling.get(),
                       self._new_check_fatigue.get(), self._new_check_slamming.get(),
-                      self._new_check_local_buckling.get(), False)
+                      self._new_check_local_buckling.get(), False, self._new_check_buckling_ml_cl.get(), False)
 
         self.pso_parameters = (self._new_swarm_size.get(), self._new_omega.get(), self._new_phip.get(),
                                self._new_phig.get(),self._new_maxiter.get(), self._new_minstep.get(),
@@ -538,6 +614,7 @@ class CreateOptGeoWindow():
         slamming_press = [list() for dummy in range(7)]
 
         broke = False
+        pressure_side = 'both sides'  # default value
         for line,coord in self._opt_structure.items():
             if self.opt_create_struc_obj(self._opt_structure[line]) is None:
                 broke = True
@@ -547,9 +624,10 @@ class CreateOptGeoWindow():
                 fat_obj_single = self.opt_create_struc_obj(self._opt_structure[line])[2]
                 fatigue_objects.append(fat_obj_single)
 
+
             if __name__ == '__main__':
                 import ANYstructure_local.example_data as ex
-                lateral_press.append(200)  # for testing
+                lateral_press.append(0.2)  # for testing
                 slamming_press.append(0)
                 fatigue_objects.append(ex.get_fatigue_object())
                 for pressure in ex.get_geo_opt_fat_press():
@@ -557,12 +635,24 @@ class CreateOptGeoWindow():
                                                pressure['p_ext']['part']),
                                               (pressure['p_int']['loaded'], pressure['p_int']['ballast'],
                                                pressure['p_int']['part'])))
+                pressure_side = 'both sides'
 
             else:
                 p1, p2 = self._opt_structure[line]
-                closet_line = self.opt_find_closest_orig_line([(p2[0]-p1[0])*0.5, (p2[1]-p1[1])*0.5])
+                # Check if line is horizontal or vertical
+                if p2[0] == p1[0]: # Vertical
+                    to_find = [p2[0], min(p2[1], p1[1]) + abs((p2[1]-p1[1])*0.5)]
+                elif p2[1] == p1[1]: # Horizontal
+                    to_find = [min(p2[0], p1[0])+ (p2[0]-p1[0])*0.5, p2[1]]
+                else: # Other orientations
+                    to_find = [min(p2[0],p1[0])+abs((p2[0]-p1[0])*0.5), min(p2[1]-p1[1])+abs((p2[1]-p1[1])*0.5)]
+
+                # Taking properites from the closest line.
+                closet_line = self.opt_find_closest_orig_line(to_find)
+                pressure_side = self._line_to_struc[closet_line].overpressure_side
+                #print('Closest line', closet_line, p1, p2, to_find)
                 gotten_lat_press = self.app.get_highest_pressure(closet_line)
-                lateral_press.append(gotten_lat_press['normal'] / 1000)
+                lateral_press.append(gotten_lat_press['normal'] / 1e6)
                 slamming_press.append(gotten_lat_press['slamming'])
                 if fat_obj_single is not None:
                     fat_press_single = self.app.get_fatigue_pressures(closet_line, fat_obj_single.get_accelerations())
@@ -582,27 +672,65 @@ class CreateOptGeoWindow():
                                 message='This field cannot be subdivided or there are no loads. Error.')
             return None
 
-        # found_files = self._filez
-        # if self._toggle_btn.config('relief')[-1] == 'sunken':
-        #     found_files, predefined_stiffener_iter = self.toggle(found_files=found_files, iterating=True)
-        # else:
-        #     predefined_stiffener_iter = None
-
-
         if not load_pre:
+            min_var = self.get_lower_bounds()
+            max_var = self.get_upper_bounds()
+            deltas = self.get_deltas()
+            spacings = np.arange(min_var[0], max_var[0],
+                                 deltas[0])
+            resulting_geo = list()
+
+
+            if self._new_harmonize_spacing.get():
+                geo_results = dict()
+                for spacing in spacings:
+                    this_min_var = copy.deepcopy(min_var)
+                    this_min_var[0] = spacing
+                    this_max_var = copy.deepcopy(max_var)
+                    this_max_var[0] = spacing
+
+                    geo_results = op.run_optmizataion(initial_structure_obj=init_objects,min_var=this_min_var,
+                                                      max_var=this_max_var,lateral_pressure=lateral_press,
+                                                      deltas=self.get_deltas(), algorithm='anysmart',side=pressure_side,
+                                                      const_chk = contraints,pso_options = self.pso_parameters,
+                                                      is_geometric=True,fatigue_obj= fatigue_objects,
+                                                      fat_press_ext_int=fat_press_ext_int,
+                                                      min_max_span=min_max_span, tot_len=self.opt_get_length(),
+                                                      frame_height=self.opt_get_distance(), frame_distance = distances,
+                                                      predefined_stiffener_iter=self._filez,
+                                                      processes = self._new_processes.get(),
+                                                      slamming_press=slamming_press, opt_girder_prop=opt_girder_prop,
+                                                      fdwn = self._new_fdwn.get(), fup = self._new_fdwn.get(),
+                                                      ml_algo=self._ML_buckling)
+                    resulting_geo.append(geo_results)
+
+                #need to find the lowest
+                for fraction in resulting_geo[0].keys():
+                    weight = float('inf')
+                    best_idx = None
+                    for idx, geo_res in enumerate(resulting_geo):
+                        this_sub_fraction_weight = geo_res[fraction][0]
+                        if this_sub_fraction_weight < weight:
+
+                            best_idx = idx
+                            weight = this_sub_fraction_weight
+
+                    geo_results[fraction] = resulting_geo[best_idx][fraction]
+
+            else:
+                geo_results = op.run_optmizataion(initial_structure_obj=init_objects, min_var=self.get_lower_bounds(),
+                                                  max_var=self.get_upper_bounds(), lateral_pressure=lateral_press,
+                                                  deltas=self.get_deltas(), algorithm='anysmart', side=pressure_side,
+                                                  const_chk=contraints, pso_options=self.pso_parameters,
+                                                  is_geometric=True, fatigue_obj=fatigue_objects,
+                                                  fat_press_ext_int=fat_press_ext_int,
+                                                  min_max_span=min_max_span, tot_len=self.opt_get_length(),
+                                                  frame_height=self.opt_get_distance(), frame_distance=distances,
+                                                  predefined_stiffener_iter=self._filez,
+                                                  processes=self._new_processes.get(),
+                                                  slamming_press=slamming_press, opt_girder_prop=opt_girder_prop,
+                                                  fdwn=self._new_fdwn.get(), fup=self._new_fdwn.get())
 
-            geo_results = op.run_optmizataion(initial_structure_obj=init_objects,min_var=self.get_lower_bounds(),
-                                              max_var=self.get_upper_bounds(),lateral_pressure=lateral_press,
-                                              deltas=self.get_deltas(), algorithm='anysmart',side='p',
-                                              const_chk = contraints,pso_options = self.pso_parameters,
-                                              is_geometric=True,fatigue_obj= fatigue_objects,
-                                              fat_press_ext_int=fat_press_ext_int,
-                                              min_max_span=min_max_span, tot_len=self.opt_get_length(),
-                                              frame_height=self.opt_get_distance(), frame_distance = distances,
-                                              predefined_stiffener_iter=self._filez,
-                                              processes = self._new_processes.get(),
-                                              slamming_press=slamming_press, opt_girder_prop=opt_girder_prop,
-                                              fdwn = self._new_fdwn.get(), fup = self._new_fdwn.get())
             self._geo_results = geo_results
 
             if len([val*2 for val in self._geo_results.keys()]) != 0:
@@ -611,10 +739,11 @@ class CreateOptGeoWindow():
                                                            *tuple([val*2 for val in self._geo_results.keys()]),
                                                            command=self.get_plate_field_options)
                 self._ent_option_fractions.place(x=self._option_fractions_place[0], y=self._option_fractions_place[1])
-            # #SAVING RESULTS
-            # if save_results:
-            #     with open('geo_opt_2.pickle', 'wb') as file:
-            #         pickle.dump(geo_results, file)
+
+            #SAVING RESULTS
+            if save_results:
+                with open('geo_opt_2.pickle', 'wb') as file:
+                    pickle.dump(geo_results, file)
         else:
             with open('geo_opt_2.pickle', 'rb') as file:
                 self._geo_results = pickle.load(file)
@@ -633,9 +762,17 @@ class CreateOptGeoWindow():
             else:
                 filename = save_file.name
 
-        save_file = self.draw_result_text(self._geo_results, save_to_file=filename)
+        save_file, xplot, yplot = self.draw_result_text(self._geo_results, save_to_file=filename)
         self.draw_select_canvas(opt_results=self._geo_results, save_file = save_file)
 
+        plt.axes(facecolor='lightslategray')
+        plt.plot(xplot, yplot,color='yellow', linestyle='solid', marker='o',markerfacecolor='white', markersize=6)
+        plt.xlabel('Length of plate fields [m]')
+        plt.ylabel('Weight / max weight')
+        plt.title('Length of plate fields vs. total weight')
+        plt.grid()
+        plt.show()
+
     def opt_get_fractions(self):
         ''' Finding initial number of fractions '''
         init_fractions = 0
@@ -663,8 +800,8 @@ class CreateOptGeoWindow():
             return None
         objects = [copy.deepcopy(x) if x != None else None for x in
                    self._line_to_struc[self.opt_find_closest_orig_line(point)]]
-
-        objects[0].set_span(dist(pt1,pt2))
+        objects[0].Plate.set_span(dist(pt1,pt2))
+        objects[0].Stiffener.set_span(dist(pt1, pt2))
 
         return objects
 
@@ -684,7 +821,7 @@ class CreateOptGeoWindow():
                 current[0] += (vector[0]/distance) * delta
                 current[1] += (vector[1]/distance) * delta
                 if dist(coord,current) <= 0.1:
-                    if self._line_to_struc[key][0].get_structure_type() not in ('GENERAL_INTERNAL_NONWT', 'FRAME'):
+                    if self._line_to_struc[key][0].Plate.get_structure_type() not in ('GENERAL_INTERNAL_NONWT', 'FRAME'):
                         return key
                     else:
                         return None
@@ -830,12 +967,16 @@ class CreateOptGeoWindow():
         Estimate the running time of the algorithm.
         :return:
         '''
-        pass
+
         # try:
         #     self._runnig_time_label.config(text=str(self.get_running_time()))
         # except ZeroDivisionError:
         #     pass  # _tkinter.TclError: pass
 
+        if self._new_check_buckling_ml_cl.get():
+            self._new_check_buckling.set(False)
+            self._new_check_local_buckling.set(False)
+
     def get_upper_bounds(self):
         '''
         Return an numpy array of upper bounds.
@@ -981,20 +1122,21 @@ class CreateOptGeoWindow():
 
         if line != None:
             if __name__ == '__main__':
-                lateral_press = 200  # for testing
+                lateral_press = 0.2  # for testing
             else:
-                lateral_press = self.app.get_highest_pressure(line)['normal'] / 1000
+                lateral_press = self.app.get_highest_pressure(line)['normal'] /1e6
             self._canvas_opt.create_text(250, self._prop_canvas_dim[1] - 10,
                                          text='Lateral pressure: ' + str(lateral_press) + ' kPa',
                                          font='Verdana 10 bold', fill='red')
 
-    def draw_select_canvas(self, load_selected=False, opt_results = None, save_file = None):
+    def draw_select_canvas(self, opt_results = None, save_file = None):
         '''
         Making the lines canvas.
         :return:
         '''
         self._canvas_select.delete('all')
         text_type = 'Verdana 8'
+
         if opt_results is None:
             # stippled lines and text.
 
@@ -1021,7 +1163,7 @@ class CreateOptGeoWindow():
                     coord2 = self.get_point_canvas_coord('point' + str(value[1]))
                     vector = [coord2[0] - coord1[0], coord2[1] - coord1[1]]
                     # drawing a bold line if it is selected
-                    if self._line_to_struc[line][0].get_structure_type() not in ('GENERAL_INTERNAL_NONWT','FRAME'):
+                    if self._line_to_struc[line][0].Plate.get_structure_type() not in ('GENERAL_INTERNAL_NONWT','FRAME'):
 
                         if line in self._active_lines:
                             self._canvas_select.create_line(coord1, coord2, width=6, fill=color,stipple='gray50')
@@ -1056,6 +1198,9 @@ class CreateOptGeoWindow():
                     vector = [coord2[0] - coord1[0], coord2[1] - coord1[1]]
                     self._canvas_select.create_line(coord1, coord2, width=6, fill=color)
 
+                    # self._canvas_select.create_polygon(points, outline='#f11',
+                    #                       fill='#1f1', width=2)
+
             # drawing the point dictionary
 
             for key,value in self._point_dict.items():
@@ -1107,8 +1252,7 @@ class CreateOptGeoWindow():
                 #     y_loc = 40
 
                 y_loc = y_loc + delta
-
-                check_ok = [val[3] is True for val in opt_results[key][1]]
+                check_ok = [val[2] is True for val in values[1]]
 
                 if save_file is not None:
                     save_file.write('\n')
@@ -1123,7 +1267,8 @@ class CreateOptGeoWindow():
 
                 for data_idx, data in enumerate(values[1]):
                     for idx, stuc_info in enumerate(data):
-                        if type(stuc_info) == ANYstructure.calc_structure.Structure:
+                        if type(stuc_info) == ANYstructure_local.calc_structure.AllStructure:
+
                             if y_loc > 700:
                                 y_loc = 120
                                 start_x += 350
@@ -1140,13 +1285,15 @@ class CreateOptGeoWindow():
                             elif item_count == len(values[1])-1:
                                 endstring = ' -END 2-'+' OK!\n' if values[1][data_idx][3] else ' -END 2-'+' NOT OK!\n'
                             self._canvas_select.create_text([start_x + delta, y_loc],
-                                                            text=stuc_info.get_one_line_string()+endstring,
+                                                            text=stuc_info.get_one_line_string_mixed()+endstring,
                                                             anchor='w', font=text_type)
                             y_loc += 15
 
                             if save_file is not None:
-                                save_file.write(stuc_info.get_one_line_string()+' ' + stuc_info.get_extended_string() +
-                                                ' | ' + stuc_info.get_report_stresses() + endstring)
+                                save_file.write(stuc_info.get_one_line_string_mixed()+' ' +
+                                                stuc_info.get_extended_string_mixed() +
+                                                ' | ' + stuc_info.Plate.get_report_stresses() +
+                                                endstring)
                             item_count += 1
 
                 if save_file is not None:
@@ -1196,7 +1343,6 @@ class CreateOptGeoWindow():
                 max_weight = weight if weight > max_weight else max_weight
 
 
-
         if save_to_file is not None:
             save_file = open(save_to_file, 'w')
             save_file.write('| Plate fields | Fields length | Weight index | All OK? |\n')
@@ -1204,43 +1350,40 @@ class CreateOptGeoWindow():
 
         for key, value in self._geo_results.items():
             y_loc = y_loc + delta
-            check_ok = [val[3] is True for val in self._geo_results[key][1]]
+
+            check_ok = [val[2] is True for val in value[1]]
 
             self._canvas_opt.create_text([start_x + 20, y_loc ], text=str(len(check_ok)),
                                          anchor='w', font=text_type)
 
             self._canvas_opt.create_text([start_x + 120, y_loc ], text=str('No results\n' if
                                                                            self._geo_results[key][1][0][0] is None else
-                                                                           round(self._geo_results[key][1][0][0]
-                                                                                 .get_span(),4)),
+                                                                           round(self._geo_results[key][1][0][0].
+                                                                                 Plate.get_span(),4)),
                                          anchor='w', font=text_type)
             self._canvas_opt.create_text([start_x + 220, y_loc ],
                                          text=str(round(self._geo_results[key][0] / max_weight, 3))
                                          if max_weight != 0 else '',
                                          anchor='w', font=text_type)
+
             self._canvas_opt.create_text([start_x + 330, y_loc ], text=str(all(check_ok)),
                                          anchor='w', font=text_type)
 
             if save_to_file is not None:
                 save_file.write(str(len(check_ok))+ ' ' + 'No results\n' if self._geo_results[key][1][0][0] is None
-                                                             else str(round(self._geo_results[key][1][0][0].get_span(),
+                                                             else str(round(self._geo_results[key][1][0][0].Plate.get_span(),
                                                                             4)) + ' ' +
                                                                   str(round(self._geo_results[key][0] / max_weight, 3))
                                                                   + '\n' if max_weight != 0 else
                 '' + ' ' + str(all(check_ok))+'\n')
             if self._geo_results[key][1][0][0] is not None:
-                xplot.append(round(self._geo_results[key][1][0][0].get_span(),4))
+                xplot.append(round(self._geo_results[key][1][0][0].Plate.get_span(),4))
                 yplot.append(round(self._geo_results[key][0] / max_weight, 4))
 
-        plt.axes(facecolor='lightslategray')
-        plt.plot(xplot, yplot,color='yellow', linestyle='solid', marker='o',markerfacecolor='white', markersize=6)
-        plt.xlabel('Length of plate fields [m]')
-        plt.ylabel('Weight / max weight')
-        plt.title('Length of plate fields vs. total weight')
-        plt.grid()
-        plt.show()
-        if save_to_file:
-            return save_file
+        if save_to_file is not None:
+            return save_file, xplot, yplot
+        else:
+            return None, xplot, yplot
 
     def algorithm_info(self):
         ''' When button is clicked, info is displayed.'''
@@ -1365,12 +1508,12 @@ class CreateOptGeoWindow():
         method is referenced in
         '''
         self._previous_drag_mouse = [event.x, event.y]
-        if type(self._geo_results) is not list():
-            if self._geo_results is not None:
-                return
-        else:
-            if self._geo_results[0] is not None:
-                return
+        # if type(self._geo_results) is not list():
+        #     if self._geo_results is not None:
+        #         return
+        # else:
+        #     if self._geo_results[0] is not None:
+        #         return
         click_x = self._canvas_select.winfo_pointerx() - self._canvas_select.winfo_rootx()
         click_y = self._canvas_select.winfo_pointery() - self._canvas_select.winfo_rooty()
 
@@ -1471,7 +1614,7 @@ class CreateOptGeoWindow():
             return
         self._frame.destroy()
 
-    def toggle(self, found_files = None, obj = None, iterating = False):
+    def toggle(self, found_files = None, obj = None, iterating = False, given_path: str = None):
         '''
         On off button.
         :param found_files:
@@ -1496,13 +1639,23 @@ class CreateOptGeoWindow():
             self._ent_spacing_upper.config(bg = 'lightgreen')
             self._ent_spacing_lower.config(bg = 'lightgreen')
             self._ent_delta_spacing.config(bg = 'lightgreen')
-            self._filez = list(askopenfilenames(parent=self._frame, title='Choose files to open'))
+            self._ent_pl_thk_upper.config(bg = 'lightgreen')
+            self._ent_pl_thk_lower.config(bg = 'lightgreen')
+            self._ent_delta_pl_thk.config(bg = 'lightgreen')
+            if given_path is None:
+                self._filez = list(askopenfilenames(parent=self._frame, title='Choose files to open',
+                                                    initialdir=self._root_dir))
+            else:
+                self._filez = [given_path]
             if self._filez == []:
                 self._toggle_btn.config(relief="raised")
                 self._toggle_btn.config(bg='salmon')
                 self._ent_spacing_upper.config(bg='white')
                 self._ent_spacing_lower.config(bg='white')
                 self._ent_delta_spacing.config(bg='white')
+                self._ent_pl_thk_upper.config(bg='white')
+                self._ent_pl_thk_lower.config(bg='white')
+                self._ent_delta_pl_thk.config(bg='white')
 
         return found_files, predefined_structure
 
@@ -1516,8 +1669,7 @@ class CreateOptGeoWindow():
     def plot_results(self):
         'Plotting a selected panel'
         if self._geo_results is not None \
-                and type(self._new_option_fraction.get()) == int \
-                and type(self._new_option_panel.get()) == int:
+                and type(self._new_option_fraction.get()) == int and type(self._new_option_panel.get()) == int:
             op.plot_optimization_results(self._geo_results[int(self._new_option_fraction.get()/2)][1]
                                          [self._new_option_panel.get()])
 
@@ -1541,9 +1693,11 @@ class CreateOptGeoWindow():
                                   self._canvas_draw_origo[1]-(self._previous_drag_mouse[1]-event.y))
 
         self._previous_drag_mouse = (event.x,event.y)
+
         self.draw_select_canvas()
 
 if __name__ == '__main__':
     root = tk.Tk()
     my_app = CreateOptGeoWindow(master=root)
+
     root.mainloop()
