@@ -1,7 +1,8 @@
 # This is where the optimization is done.
 import tkinter as tk
 from _tkinter import TclError
-
+from ANYstructure_local.calc_structure import CalcScantlings, AllStructure
+import ANYstructure_local.example_data as ex
 import ANYstructure_local.optimize as op
 import numpy as np
 import time, os, datetime
@@ -19,19 +20,74 @@ class CreateOptimizeWindow():
     def __init__(self,master,app=None):
         super(CreateOptimizeWindow,self).__init__()
         if __name__ == '__main__':
-            self._initial_structure_obj = test.get_structure_calc_object(heavy=True)
-            self._initial_calc_obj = test.get_structure_calc_object(heavy=True)
-            self._lateral_pressure = 200
+            import pickle
+            Plate = CalcScantlings(ex.obj_dict)
+            Stiffener = None#CalcScantlings(ex.obj_dict)
+            Girder = None  # CalcScantlings(ex.obj_dict_heavy)
+            self._initial_calc_obj  = AllStructure(Plate=Plate, Stiffener=Stiffener, Girder=Girder,
+                                  main_dict=ex.prescriptive_main_dict)
+
+            #self._initial_calc_obj = test.get_structure_calc_object(heavy=True)
+            self._lateral_pressure = 0.2
             self._fatigue_object = test.get_fatigue_object()
             self._fatigue_pressure = test.get_fatigue_pressures()
             self._slamming_pressure = test.get_slamming_pressure()
             image_dir = os.path.dirname(__file__)+'\\images\\'
             self._PULS_object = None
             self._puls_acceptance = 0.87
+            self._initial_calc_obj.lat_press = self._lateral_pressure/1000
+            self._ML_buckling = dict()  # Buckling machine learning algorithm
+            self._root_dir = 'C:\\Github\\ANYstructure\\ANYstructure_local\\'
+            for name, file_base in zip(['cl SP buc int predictor', 'cl SP buc int scaler',
+                                        'cl SP ult int predictor', 'cl SP ult int scaler',
+                                        'cl SP buc GLGT predictor', 'cl SP buc GLGT scaler',
+                                        'cl SP ult GLGT predictor', 'cl SP ult GLGT scaler',
+                                        'cl UP buc int predictor', 'cl UP buc int scaler',
+                                        'cl UP ult int predictor', 'cl UP ult int scaler',
+                                        'cl UP buc GLGT predictor', 'cl UP buc GLGT scaler',
+                                        'cl UP ult GLGT predictor', 'cl UP ult GLGT scaler'
+                                        ],
+                                       ["CL_output_cl_buc_predictor_In-plane_support_cl_1_SP",
+                                        "CL_output_cl_buc_scaler_In-plane_support_cl_1_SP",
+                                        "CL_output_cl_ult_predictor_In-plane_support_cl_1_SP",
+                                        "CL_output_cl_ult_scaler_In-plane_support_cl_1_SP",
+                                        "CL_output_cl_buc_predictor_In-plane_support_cl_2,_3_SP",
+                                        "CL_output_cl_buc_scaler_In-plane_support_cl_2,_3_SP",
+                                        "CL_output_cl_ult_predictor_In-plane_support_cl_2,_3_SP",
+                                        "CL_output_cl_ult_scaler_In-plane_support_cl_2,_3_SP",
+                                        "CL_output_cl_buc_predictor_In-plane_support_cl_1_UP",
+                                        "CL_output_cl_buc_scaler_In-plane_support_cl_1_UP",
+                                        "CL_output_cl_ult_predictor_In-plane_support_cl_1_UP",
+                                        "CL_output_cl_ult_scaler_In-plane_support_cl_1_UP",
+                                        "CL_output_cl_buc_predictor_In-plane_support_cl_2,_3_UP",
+                                        "CL_output_cl_buc_scaler_In-plane_support_cl_2,_3_UP",
+                                        "CL_output_cl_ult_predictor_In-plane_support_cl_2,_3_UP",
+                                        "CL_output_cl_ult_scaler_In-plane_support_cl_2,_3_UP",
+                                        "CL_CSR-Tank_req_cl_predictor",
+                                        "CL_CSR-Tank_req_cl_UP_scaler",
+                                        "CL_CSR_plate_cl,_CSR_web_cl,_CSR_web_flange_cl,_CSR_flange_cl_predictor",
+                                        "CL_CSR_plate_cl,_CSR_web_cl,_CSR_web_flange_cl,_CSR_flange_cl_SP_scaler"]):
+                self._ML_buckling[name] = None
+                if os.path.isfile(file_base + '.pickle'):
+                    file = open(file_base + '.pickle', 'rb')
+                    from sklearn.neural_network import MLPClassifier
+                    from sklearn.preprocessing import StandardScaler
+                    self._ML_buckling[name] = pickle.load(file)
+                    file.close()
+
+            self._ML_classes = {0: 'N/A',
+                                1: 'A negative utilisation factor is found.',
+                                2: 'At least one of the in-plane loads must be non-zero.',
+                                3: 'Division by zero',
+                                4: 'Overflow',
+                                5: 'The aspect ratio exceeds the PULS code limit',
+                                6: 'The global slenderness exceeds 4. Please reduce stiffener span or increase stiffener height.',
+                                7: 'The applied pressure is too high for this plate field.', 8: 'web-flange-ratio',
+                                9: 'UF below or equal 0.87', 10: 'UF between 0.87 and 1.0', 11: 'UF above 1.0'}
         else:
             self.app = app
-            self._initial_structure_obj = app._line_to_struc[app._active_line][0]
-            self._initial_calc_obj = app._line_to_struc[app._active_line][1]
+
+            self._initial_calc_obj = app._line_to_struc[app._active_line][0]
             self._fatigue_object = app._line_to_struc[app._active_line][2]
             try:
                 self._fatigue_pressure = app.get_fatigue_pressures(app._active_line,
@@ -39,7 +95,7 @@ class CreateOptimizeWindow():
             except AttributeError:
                 self._fatigue_pressure = None
             try:
-                self._lateral_pressure = self.app.get_highest_pressure(self.app._active_line)['normal'] / 1000
+                self._lateral_pressure = self.app.get_highest_pressure(self.app._active_line)['normal'] / 1e6
             except KeyError:
                 self._lateral_pressure = 0
             try:
@@ -55,6 +111,8 @@ class CreateOptimizeWindow():
             self._PULS_object = app._PULS_results
             self._puls_acceptance = self.app._new_puls_uf.get()
 
+            self._ML_buckling = app._ML_buckling
+
         self._predefined_stiffener_iter = None
 
         self._frame = master
@@ -81,12 +139,20 @@ class CreateOptimizeWindow():
 
         tk.Label(self._frame,text='-- Structural optimizer --',font='Verdana 15 bold').place(x=10,y=10)
 
-        self._spacing = self._initial_structure_obj.get_s()
-        self._pl_thk = self._initial_structure_obj.get_pl_thk()
-        self._stf_web_h = self._initial_structure_obj.get_web_h()
-        self._stf_web_thk =self._initial_structure_obj.get_web_thk()
-        self._fl_w = self._initial_structure_obj.get_fl_w()
-        self._fl_thk =self._initial_structure_obj.get_fl_thk()
+        if self._initial_calc_obj.Stiffener is not None:
+            self._spacing = self._initial_calc_obj.Plate.get_s()
+            self._pl_thk = self._initial_calc_obj.Plate.get_pl_thk()
+            self._stf_web_h = self._initial_calc_obj.Stiffener.get_web_h()
+            self._stf_web_thk =self._initial_calc_obj.Stiffener.get_web_thk()
+            self._fl_w = self._initial_calc_obj.Stiffener.get_fl_w()
+            self._fl_thk =self._initial_calc_obj.Stiffener.get_fl_thk()
+        else:
+            self._spacing = self._initial_calc_obj.Plate.get_s()
+            self._pl_thk = self._initial_calc_obj.Plate.get_pl_thk()
+            self._stf_web_h = 0
+            self._stf_web_thk =0
+            self._fl_w = 0
+            self._fl_thk =0
 
         # upper and lower bounds for optimization
         #[0.6, 0.012, 0.3, 0.01, 0.1, 0.01]
@@ -223,7 +289,8 @@ class CreateOptimizeWindow():
         tk.Label(self._frame, text='Flange thk. [mm]', font='Verdana 7 bold').place(x=start_x + 6.97 * dx, y=start_y-0.6*dy)
         tk.Label(self._frame, text='--------- Number of combinations to run --------->\n'
                                    'PULS buckling is time consuming, about 0.2 sec. per comb.\n'
-                                   'RP-C203 is much faster and can run many more combinations, 1M+. ',
+                                   'RP-C203 is much faster and can run many more combinations, 1M+.\n'
+                                   'ML-CL is about as fast as RP-C203.',
                  font='Verdana 9 bold').place(x=start_x+0.1*dx, y=start_y + 2.8 * dy, anchor = tk.NW)
 
         self._runnig_time_label = tk.Label(self._frame, text='',font='Verdana 12 bold', fg = 'red')
@@ -361,10 +428,11 @@ class CreateOptimizeWindow():
         self._new_delta_web_thk.set(init_thk)
         self._new_delta_fl_w.set(init_dim)
         self._new_delta_fl_thk.set(init_thk)
-        self._new_trans_stress_high.set(self._initial_structure_obj.get_sigma_y1())
-        self._new_trans_stress_low.set(self._initial_structure_obj.get_sigma_y2())
-        self._new_axial_stress.set(self._initial_structure_obj.get_sigma_x())
-        self._new_shear_stress.set(self._initial_structure_obj.get_tau_xy())
+        self._new_trans_stress_high.set(self._initial_calc_obj.Plate.get_sigma_y1())
+        self._new_trans_stress_low.set(self._initial_calc_obj.Plate.get_sigma_y2())
+        self._new_axial_stress.set(self._initial_calc_obj.Plate.get_sigma_x1())
+
+        self._new_shear_stress.set(self._initial_calc_obj.Plate.get_tau_xy())
 
         self._new_design_pressure.set(self._lateral_pressure)
         self._new_slamming_pressure.set(self._slamming_pressure)
@@ -382,11 +450,13 @@ class CreateOptimizeWindow():
         self._new_web_h_lower.set(round(max(self._stf_web_h*1000-100,100),5))
         self._new_web_thk_upper.set(round(self._stf_web_thk*1000+10,5))
         self._new_web_thk_lower.set(round(max(self._stf_web_thk*1000-10,float(10)),5))
-        if self._initial_structure_obj.get_stiffener_type() != 'FB':
-            self._new_fl_w_upper.set(min(round(self._fl_w*1000+100,5), 200))
-            self._new_fl_w_lower.set(round(max(self._fl_w*1000-100,100),5))
-            self._new_fl_thk_upper.set(round(self._fl_thk*1000+10,15))
-            self._new_fl_thk_lower.set(round(max(self._fl_thk*1000-10,10),15))
+        if self._initial_calc_obj.Stiffener is not None:
+            if self._initial_calc_obj.Stiffener.get_stiffener_type() != 'FB':
+                self._new_fl_w_upper.set(min(round(self._fl_w*1000+100,5), 200))
+                self._new_fl_w_lower.set(round(max(self._fl_w*1000-100,100),5))
+                self._new_fl_thk_upper.set(round(self._fl_thk*1000+10,15))
+                self._new_fl_thk_lower.set(round(max(self._fl_thk*1000-10,10),15))
+
         else:
             self._new_fl_w_upper.set(0)
             self._new_fl_w_lower.set(0)
@@ -395,7 +465,7 @@ class CreateOptimizeWindow():
 
         self._new_pressure_side.set('p')
         self._new_width_lg.set(10)
-        self._new_span.set(round(self._initial_structure_obj.get_span(),5))
+        self._new_span.set(round(self._initial_calc_obj.Plate.get_span(),5))
         self._new_algorithm.set('anysmart')
         self._new_algorithm_random_trials.set(100000)
         self._new_swarm_size.set(100)
@@ -472,6 +542,7 @@ class CreateOptimizeWindow():
         self._new_check_shear_area = tk.BooleanVar()
         self._new_check_buckling = tk.BooleanVar()
         self._new_check_buckling_puls = tk.BooleanVar()
+        self._new_check_buckling_ml_cl = tk.BooleanVar()
         self._new_check_fatigue = tk.BooleanVar()
         self._new_check_slamming = tk.BooleanVar()
         self._new_check_local_buckling = tk.BooleanVar()
@@ -485,10 +556,13 @@ class CreateOptimizeWindow():
         self._new_check_local_buckling.set(True)
         self._new_use_weight_filter.set(True)
         self._new_check_buckling_puls.set(False)
+        self._new_check_buckling_ml_cl.set(False)
         self._new_check_buckling_puls.trace('w', self.update_running_time)
+        self._new_check_buckling_ml_cl.trace('w', self.update_running_time)
+
 
         start_y = 140
-        tk.Label(self._frame,text='Check for minimum section modulus').place(x=start_x+dx*9.7,y=start_y+4*dy)
+        tk.Label(self._frame,text='Check for minimum section mofdulus').place(x=start_x+dx*9.7,y=start_y+4*dy)
         tk.Label(self._frame, text='Check for minimum plate thk.').place(x=start_x+dx*9.7,y=start_y+5*dy)
         tk.Label(self._frame, text='Check for minimum shear area').place(x=start_x+dx*9.7,y=start_y+6*dy)
         tk.Label(self._frame, text='Check for buckling (RP-C201)').place(x=start_x+dx*9.7,y=start_y+7*dy)
@@ -497,6 +571,7 @@ class CreateOptimizeWindow():
         tk.Label(self._frame, text='Check for local stf. buckling').place(x=start_x + dx * 9.7, y=start_y + 10 * dy)
         tk.Label(self._frame, text='Use weight filter (for speed)').place(x=start_x + dx * 9.7, y=start_y + 11 * dy)
         tk.Label(self._frame, text='Check for buckling (PULS)').place(x=start_x + dx * 9.7, y=start_y + 12 * dy)
+        tk.Label(self._frame, text='Check for buckling (ML-CL)').place(x=start_x + dx * 9.7, y=start_y + 13 * dy)
 
         tk.Checkbutton(self._frame,variable=self._new_check_sec_mod).place(x=start_x+dx*12,y=start_y+4*dy)
         tk.Checkbutton(self._frame, variable=self._new_check_min_pl_thk).place(x=start_x+dx*12,y=start_y+5*dy)
@@ -510,6 +585,8 @@ class CreateOptimizeWindow():
                                                                                    y=start_y + 11 * dy)
         tk.Checkbutton(self._frame, variable=self._new_check_buckling_puls).place(x=start_x + dx * 12,
                                                                                    y=start_y + 12 * dy)
+        tk.Checkbutton(self._frame, variable=self._new_check_buckling_ml_cl).place(x=start_x + dx * 12,
+                                                                                   y=start_y + 13 * dy)
 
         # Stress scaling
         self._new_fup = tk.DoubleVar()
@@ -531,7 +608,7 @@ class CreateOptimizeWindow():
         self._toggle_btn = tk.Button(self._frame, text="Iterate predefiened stiffeners", relief="raised",
                                      command=self.toggle, bg = 'salmon')
         self._toggle_btn.place(x=start_x, y=start_y - dy * 2)
-        self._toggle_object, self._filez = self._initial_structure_obj, None
+        self._toggle_object, self._filez = self._initial_calc_obj, None
         self.draw_properties()
         self.update_running_time()
 
@@ -642,8 +719,9 @@ class CreateOptimizeWindow():
         contraints = (self._new_check_sec_mod.get(),self._new_check_min_pl_thk.get(),
                       self._new_check_shear_area.get(), self._new_check_buckling.get(),
                       self._new_check_fatigue.get(), self._new_check_slamming.get(),
-                      self._new_check_local_buckling.get(), self._new_check_buckling_puls.get())
-        self._initial_structure_obj.set_span(self._new_span.get())
+                      self._new_check_local_buckling.get(), self._new_check_buckling_puls.get(),
+                      self._new_check_buckling_ml_cl.get(), False)
+        self._initial_calc_obj.Plate.set_span(self._new_span.get())
 
         if self._fatigue_pressure is not None:
 
@@ -654,7 +732,7 @@ class CreateOptimizeWindow():
         else:
             fat_press = None
 
-        self._opt_results= op.run_optmizataion(self._initial_structure_obj,self.get_lower_bounds(),
+        self._opt_results= op.run_optmizataion(self._initial_calc_obj,self.get_lower_bounds(),
                                                self.get_upper_bounds(),self._new_design_pressure.get(),
                                                self.get_deltas(),algorithm=self._new_algorithm.get(),
                                                trials=self._new_algorithm_random_trials.get(),
@@ -668,27 +746,34 @@ class CreateOptimizeWindow():
                                                use_weight_filter = False if self._new_check_buckling_puls.get()
                                                else self._new_use_weight_filter.get(),
                                                puls_sheet = puls_sheet_location, puls_acceptance = puls_acceptance,
-                                               fdwn = self._new_fdwn.get(), fup = self._new_fdwn.get())
+                                               fdwn = self._new_fdwn.get(), fup = self._new_fdwn.get(),
+                                               ml_algo= self._ML_buckling)
 
         if self._opt_results is not None and self._opt_results[0] is not None:
             self._opt_actual_running_time.config(text='Actual running time: \n'
                                                      +str(round((time.time()-t_start)/60,4))+' min')
             self._opt_actual_running_time.update()
             self._opt_runned = True
-            self._result_label.config(text='Optimization result | Spacing: '+
-                                           str(round(self._opt_results[0].get_s(),10)*1000)+
-                                          ' Plate thickness: '+str(round(self._opt_results[0].get_plate_thk()*1000,10))+
-                                          ' Stiffener - T'+str(round(self._opt_results[0].get_web_h()*1000,10))+'x'
-                                          +str(round(self._opt_results[0].get_web_thk()*1000,10))+
-                                          '+'+str(round(self._opt_results[0].get_fl_w()*1000,10))+'x'
-                                          +str(round(self._opt_results[0].get_fl_thk()*1000,10)))
+            if self._opt_results[0].Stiffener is not None:
+                text = 'Optimization result | Spacing: ' + str(round(self._opt_results[0].Plate.get_s(), 10) * 1000) +\
+                ' Plate thickness: ' + str(round(self._opt_results[0].Plate.get_pl_thk() * 1000, 10)) +\
+                ' Stiffener - T' + str(round(self._opt_results[0].Stiffener.get_web_h() * 1000, 10)) + 'x'\
+                +str(round(self._opt_results[0].Stiffener.get_web_thk() * 1000, 10)) +\
+                '+' + str(round(self._opt_results[0].Stiffener.get_fl_w() * 1000, 10)) + 'x'\
+                +str(round(self._opt_results[0].Stiffener.get_fl_thk() * 1000, 10))
+            else:
+                text = 'Optimization result | Spacing: ' + str(round(self._opt_results[0].Plate.get_s(), 10) * 1000) +\
+                ' Plate thickness: ' + str(round(self._opt_results[0].Plate.get_pl_thk() * 1000, 10))
 
-            self._new_opt_spacing.set(round(self._opt_results[0].get_s(),5))
-            self._new_opt_pl_thk.set(round(self._opt_results[0].get_plate_thk(),5))
-            self._new_opt_web_h.set(round(self._opt_results[0].get_web_h(),5))
-            self._new_opt_web_thk.set(round(self._opt_results[0].get_web_thk(),5))
-            self._new_opt_fl_w.set(round(self._opt_results[0].get_fl_w(),5))
-            self._new_opt_fl_thk.set(round(self._opt_results[0].get_fl_thk(),5))
+            self._result_label.config(text= text)
+
+            self._new_opt_spacing.set(round(self._opt_results[0].Plate.get_s(),5))
+            self._new_opt_pl_thk.set(round(self._opt_results[0].Plate.get_pl_thk(),5))
+            if self._opt_results[0].Stiffener is not None:
+                self._new_opt_web_h.set(round(self._opt_results[0].Stiffener.get_web_h(),5))
+                self._new_opt_web_thk.set(round(self._opt_results[0].Stiffener.get_web_thk(),5))
+                self._new_opt_fl_w.set(round(self._opt_results[0].Stiffener.get_fl_w(),5))
+                self._new_opt_fl_thk.set(round(self._opt_results[0].Stiffener.get_fl_thk(),5))
             self.draw_properties()
         else:
             messagebox.showinfo(title='Nothing found', message='No better alternatives found. Modify input.\n'
@@ -713,20 +798,6 @@ class CreateOptimizeWindow():
             return int(number_of_combinations * self.running_time_per_item['PULS' if self._new_check_buckling_puls.get()
             else 'RP']), number_of_combinations
 
-            # try:
-            #     number_of_combinations = \
-            #     max((self._new_spacing_upper.get()-self._new_spacing_lower.get())/self._new_delta_spacing.get(),1)* \
-            #     max((self._new_pl_thk_upper.get()-self._new_pl_thk_lower.get())/self._new_delta_pl_thk.get(),1)*\
-            #     max((self._new_web_h_upper.get()-self._new_web_h_lower.get())/self._new_delta_web_h.get(),1)*\
-            #     max((self._new_web_thk_upper.get()-self._new_web_thk_lower.get())/self._new_delta_web_thk.get(),1)*\
-            #     max((self._new_fl_w_upper.get()-self._new_fl_w_lower.get())/self._new_delta_fl_w.get(),1)*\
-            #     max((self._new_fl_thk_upper.get()-self._new_fl_thk_lower.get())/self._new_delta_fl_thk.get(),1)
-            #
-            #     return int(number_of_combinations*self.running_time_per_item),number_of_combinations,  \
-            #            number_of_combinations*0.1 \
-            #         if not self._new_check_buckling_puls.get() else number_of_combinations*1
-            # except TclError:
-            #     return 0,0,0
         elif self._new_algorithm.get() in ['pso','random','random_no_delta']:
             try:
                 number_of_combinations = \
@@ -772,9 +843,19 @@ class CreateOptimizeWindow():
         except (ZeroDivisionError, TclError):
             pass# _tkinter.TclError: pass
 
+        if [self._new_check_buckling_ml_cl.get(),self._new_check_buckling_puls.get(),
+            self._new_check_buckling.get()].count(True) > 1:
+            tk.messagebox.showerror('You can only select one buckling type. Reselect.')
+            if self._new_check_buckling_puls.get():
+                self._new_check_buckling_puls.set(False)
+            if self._new_check_buckling_ml_cl.get():
+                self._new_check_buckling_ml_cl.set(False)
+            if self._new_check_buckling.get():
+                self._new_check_buckling.set(False)
+                self._new_check_local_buckling.set(False)
+
+
         if self._new_check_buckling_puls.get():
-            self._new_check_local_buckling.set(False)
-            self._new_check_buckling.set(False)
             if self._PULS_object is None or self._PULS_object.puls_sheet_location is None:
                 tk.messagebox.showerror('Missing PULS sheet', 'Go back to main window and set a PULS sheet location\n'
                                                     'by running one or more lines.')
@@ -800,14 +881,6 @@ class CreateOptimizeWindow():
                          self._new_fl_w_lower.get()/1000,self._new_fl_thk_lower.get()/1000,
                          self._new_span.get(), self._new_width_lg.get()])
 
-    def get_sigmas(self):
-        '''
-        Returns the stressess.
-        :return:
-        '''
-        return np.array([self._new_trans_stress_high.get(),self._new_trans_stress_low.get(),
-                         self._new_axial_stress.get(),self._new_shear_stress.get()])
-
     def checkered(self,line_distance):
         # vertical lines at an interval of "line_distance" pixel
         for x in range(line_distance, self._canvas_dim[0], line_distance):
@@ -842,7 +915,10 @@ class CreateOptimizeWindow():
         self._canvas_opt.create_rectangle(ctr_x - m * self._stf_web_thk / 2, ctr_y-m* self._pl_thk,
                                          ctr_x + m * self._stf_web_thk / 2, ctr_y - m *(self._stf_web_h+self._pl_thk)
                                          , fill=init_color, stipple=init_stipple )
-        if self._initial_structure_obj.get_stiffener_type() not in ['L', 'L-bulb']:
+        if self._initial_calc_obj.Stiffener is None:
+            return
+
+        if self._initial_calc_obj.Stiffener.get_stiffener_type() not in ['L', 'L-bulb']:
             self._canvas_opt.create_rectangle(ctr_x-m*self._fl_w/2, ctr_y-m*(self._pl_thk+self._stf_web_h),
                                              ctr_x+m*self._fl_w/2, ctr_y-m*(self._pl_thk+self._stf_web_h+self._fl_thk),
                                              fill=init_color, stipple=init_stipple)
@@ -853,46 +929,46 @@ class CreateOptimizeWindow():
 
         if self._opt_runned:
 
-            self._canvas_opt.create_rectangle(ctr_x - m * self._opt_results[0].get_s() / 2, ctr_y,
-                                             ctr_x + m * self._opt_results[0].get_s()  / 2,
-                                             ctr_y - m * self._opt_results[0].get_pl_thk(), fill=opt_color,
+            self._canvas_opt.create_rectangle(ctr_x - m * self._opt_results[0].Stiffener.get_s() / 2, ctr_y,
+                                             ctr_x + m * self._opt_results[0].Stiffener.get_s()  / 2,
+                                             ctr_y - m * self._opt_results[0].Stiffener.get_pl_thk(), fill=opt_color,
                                              stipple=opt_stippe)
 
-            self._canvas_opt.create_rectangle(ctr_x - m * self._opt_results[0].get_web_thk() / 2, ctr_y -
-                                             m * self._opt_results[0].get_pl_thk(),
-                                             ctr_x + m * self._opt_results[0].get_web_thk() / 2,
-                                             ctr_y - m * (self._opt_results[0].get_web_h() + self._opt_results[0].get_pl_thk())
+            self._canvas_opt.create_rectangle(ctr_x - m * self._opt_results[0].Stiffener.get_web_thk() / 2, ctr_y -
+                                             m * self._opt_results[0].Stiffener.get_pl_thk(),
+                                             ctr_x + m * self._opt_results[0].Stiffener.get_web_thk() / 2,
+                                             ctr_y - m * (self._opt_results[0].Stiffener.get_web_h() + self._opt_results[0].Stiffener.get_pl_thk())
                                              , fill=opt_color, stipple=opt_stippe)
-            if self._opt_results[0].get_stiffener_type() not in ['L', 'L-bulb']:
-                self._canvas_opt.create_rectangle(ctr_x - m * self._opt_results[0].get_fl_w() / 2, ctr_y
-                                                 - m * (self._opt_results[0].get_pl_thk()+ self._opt_results[0].get_web_h()),
-                                                 ctr_x + m * self._opt_results[0].get_fl_w() / 2,ctr_y -
-                                                 m * (self._opt_results[0].get_pl_thk() + self._opt_results[0].get_web_h() +
-                                                      self._opt_results[0].get_fl_thk()),
+            if self._opt_results[0].Stiffener.get_stiffener_type() not in ['L', 'L-bulb']:
+                self._canvas_opt.create_rectangle(ctr_x - m * self._opt_results[0].Stiffener.get_fl_w() / 2, ctr_y
+                                                 - m * (self._opt_results[0].Stiffener.get_pl_thk()+ self._opt_results[0].Stiffener.get_web_h()),
+                                                 ctr_x + m * self._opt_results[0].Stiffener.get_fl_w() / 2,ctr_y -
+                                                 m * (self._opt_results[0].Stiffener.get_pl_thk() + self._opt_results[0].Stiffener.get_web_h() +
+                                                      self._opt_results[0].Stiffener.get_fl_thk()),
                                                  fill=opt_color, stipple=opt_stippe)
             else:
-                self._canvas_opt.create_rectangle(ctr_x - m * self._opt_results[0].get_web_thk() / 2, ctr_y
-                                                 - m * (self._opt_results[0].get_pl_thk()+ self._opt_results[0].get_web_h()),
-                                                 ctr_x + m * self._opt_results[0].get_fl_w() ,ctr_y -
-                                                 m * (self._opt_results[0].get_pl_thk() + self._opt_results[0].get_web_h() +
-                                                      self._opt_results[0].get_fl_thk()),
+                self._canvas_opt.create_rectangle(ctr_x - m * self._opt_results[0].Stiffener.get_web_thk() / 2, ctr_y
+                                                 - m * (self._opt_results[0].Stiffener.get_pl_thk()+ self._opt_results[0].Stiffener.get_web_h()),
+                                                 ctr_x + m * self._opt_results[0].Stiffener.get_fl_w() ,ctr_y -
+                                                 m * (self._opt_results[0].Stiffener.get_pl_thk() + self._opt_results[0].Stiffener.get_web_h() +
+                                                      self._opt_results[0].Stiffener.get_fl_thk()),
                                                  fill=opt_color, stipple=opt_stippe)
 
             self._canvas_opt.create_line(10, 50, 30, 50, fill=opt_color, width=5)
-            self._canvas_opt.create_text(270,50,text='Optimized - Pl.: '+str(round(self._opt_results[0].get_s()*1000,1))
-                                                     +'x'+ str(round(self._opt_results[0].get_pl_thk()*1000,1))+
-                                                     ' Stf.: '+str(round(self._opt_results[0].get_web_h()*1000,1))+
-                                                    'x'+str(round(self._opt_results[0].get_web_thk()*1000,1))+'+'+
-                                                            str(round(self._opt_results[0].get_fl_w()*1000,1))+
-                                                    'x'+str(round(self._opt_results[0].get_fl_thk()*1000,1)),
+            self._canvas_opt.create_text(270,50,text='Optimized - Pl.: '+str(round(self._opt_results[0].Stiffener.get_s()*1000,1))
+                                                     +'x'+ str(round(self._opt_results[0].Stiffener.get_pl_thk()*1000,1))+
+                                                     ' Stf.: '+str(round(self._opt_results[0].Stiffener.get_web_h()*1000,1))+
+                                                    'x'+str(round(self._opt_results[0].Stiffener.get_web_thk()*1000,1))+'+'+
+                                                            str(round(self._opt_results[0].Stiffener.get_fl_w()*1000,1))+
+                                                    'x'+str(round(self._opt_results[0].Stiffener.get_fl_thk()*1000,1)),
                                         font = 'Verdana 8',fill = opt_color)
             self._canvas_opt.create_text(120, 70, text='Weight (per Lg width): '
-                                                      + str(int(op.calc_weight([self._opt_results[0].get_s(),
-                                                                                self._opt_results[0].get_pl_thk(),
-                                                                                self._opt_results[0].get_web_h(),
-                                                                                self._opt_results[0].get_web_thk(),
-                                                                                self._opt_results[0].get_fl_w(),
-                                                                                self._opt_results[0].get_fl_thk(),
+                                                      + str(int(op.calc_weight([self._opt_results[0].Stiffener.get_s(),
+                                                                                self._opt_results[0].Stiffener.get_pl_thk(),
+                                                                                self._opt_results[0].Stiffener.get_web_h(),
+                                                                                self._opt_results[0].Stiffener.get_web_thk(),
+                                                                                self._opt_results[0].Stiffener.get_fl_w(),
+                                                                                self._opt_results[0].Stiffener.get_fl_thk(),
                                                                                 self._new_span.get(),
                                                                                 self._new_width_lg.get()]))),
                                         font='Verdana 8', fill=opt_color)
@@ -973,12 +1049,14 @@ class CreateOptimizeWindow():
             self._ent_pl_thk_lower.config(bg = 'lightgreen')
             self._ent_delta_pl_thk.config(bg = 'lightgreen')
 
-            open_files = askopenfilenames(parent=self._frame, title='Choose files to open')
+            open_files = askopenfilenames(parent=self._frame, title='Choose files to open',
+                                          initialdir=self._root_dir)
+            # TODO for both stiffeners and girders
+
+            self._initial_calc_obj.Stiffener.t = self._initial_calc_obj.Plate.t
+            self._initial_calc_obj.Stiffener.s = self._initial_calc_obj.Plate.s
             predefined_stiffener_iter = hlp.helper_read_section_file(files=list(open_files),
-                                                                     obj=self._initial_structure_obj)
-
-
-
+                                                                     obj=self._initial_calc_obj.Stiffener)
         if predefined_stiffener_iter == []:
             self._toggle_btn.config(relief="raised")
             self._toggle_btn.config(bg = 'salmon')
